(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.wefetch = factory());
})(this, (function () { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }

    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    const isString = (target) => typeof target === 'string';
    const isFunction = (target) => typeof target === 'function';

    class WrapperPromise extends Promise {
        get task() {
            return this._task;
        }
        set task(task) {
            this._task = task;
        }
        cancel() {
            this._task.abort();
        }
    }

    function promisify(api) {
        return function (options) {
            const promise = new WrapperPromise((resolve, reject) => {
                promise.task = api(Object.assign({ success: resolve, fail: reject }, options));
            });
            return promise;
        };
    }

    function adaptor() {
        // @ts-ignore
        if (typeof wx != 'undefined') {
            // @ts-ignore
            return promisify(wx.request);
            // @ts-ignore
        }
        else if (typeof my != 'undefined') {
            // @ts-ignore
            return promisify(my.request);
            // @ts-ignore
        }
        else if (typeof swan != 'undefined') {
            // @ts-ignore
            return promisify(swan.request);
        }
        throw new Error('The current miniProgram is not supported,pls use register method instead!');
    }

    class Interceptor {
        constructor() {
            this.handles = [];
        }
        use(fulfilled, rejected) {
            this.handles.push({ fulfilled, rejected });
            return this.handles.length - 1;
        }
        eject(id) {
            if (this.handles[id]) {
                this.handles[id] = null;
            }
        }
        forEach(fn) {
            this.handles.forEach(interceptor => interceptor && fn(interceptor));
        }
    }

    var _Wefetch_defaults, _Wefetch_task;
    // 'OPTIONS' | 'GET' | 'HEAD' | 'POST' | 'PUT' | 'DELETE' | 'TRACE' | 'CONNECT'
    class Wefetch {
        constructor(option) {
            _Wefetch_defaults.set(this, void 0);
            _Wefetch_task.set(this, void 0);
            this.before = new Interceptor();
            this.after = new Interceptor();
            __classPrivateFieldSet(this, _Wefetch_defaults, option);
        }
        dispatchRequest(option) {
            const promise = __classPrivateFieldGet(this, _Wefetch_defaults).adaptor(option);
            __classPrivateFieldSet(this, _Wefetch_task, promise._task);
            return promise.then(response => response, WrapperPromise.reject);
        }
        register(api) {
            if (!isFunction(api)) {
                throw new Error(`expect a function, but go a $${api}:${typeof api}`);
            }
            __classPrivateFieldGet(this, _Wefetch_defaults).adaptor = promisify(api);
        }
        cancel() {
            __classPrivateFieldGet(this, _Wefetch_task).abort();
        }
        get task() {
            return __classPrivateFieldGet(this, _Wefetch_task);
        }
        request(option) {
            if (isString(option)) {
                option = arguments[1] || {};
                option.url = arguments[0];
            }
            const chian = [this.dispatchRequest, undefined];
            let promise = WrapperPromise.resolve(option);
            this.before.forEach(interceptor => {
                chian.unshift(interceptor.fulfilled, interceptor.rejected);
            });
            this.after.forEach(interceptor => {
                chian.push(interceptor.fulfilled, interceptor.rejected);
            });
            while (chian.length) {
                promise = promise.then(chian.shift(), chian.shift());
            }
            return promise;
        }
        get(option) {
            return this.request(Object.assign(Object.assign({}, option), { method: 'GET' }));
        }
        head(option) {
            return this.request(Object.assign(Object.assign({}, option), { method: 'HEAD' }));
        }
        post(option) {
            return this.request(Object.assign(Object.assign({}, option), { method: 'POST' }));
        }
        delete(option) {
            return this.request(Object.assign(Object.assign({}, option), { method: 'DELETE' }));
        }
        put(option) {
            return this.request(Object.assign(Object.assign({}, option), { method: 'PUT' }));
        }
        trace(option) {
            return this.request(Object.assign(Object.assign({}, option), { method: 'TRACE' }));
        }
        connect(option) {
            return this.request(Object.assign(Object.assign({}, option), { method: 'CONNECT' }));
        }
    }
    _Wefetch_defaults = new WeakMap(), _Wefetch_task = new WeakMap();

    const defaultsOption = {
        method: 'GET',
        header: {},
        timeout: 0,
        adaptor: adaptor()
    };

    // Import here Polyfills if needed. Recommended core-js (npm i -D core-js)
    function createInstance(defaultOptions) {
        let context = new Wefetch(defaultOptions);
        return context;
    }
    const wf = createInstance(defaultsOption);

    return wf;

}));
//# sourceMappingURL=wefetch.umd.js.map
